#include "gamec.h"

void do_cycle(GameControl** gamec) {

	GList* walker;
	
	if(CHK((*gamec)->game_state,GAME_FREEZE) == 0) {

		update_area_spaces(gamec);
		repaint_drawing_area(gamec);

		if(CHK((*gamec)->game_state,MONSTERS_FREEZE) == 0) {

			walker = (*gamec)->monsters;

			while(walker != NULL) {

				if(walker->data != NULL) {
			
					(*gamec)->turn = ((Monster*)walker->data);

					if(CHK((*gamec)->turn->properties,IS_PLAYER)) {
	
						parse_key_stack(gamec);
						area_center_monster(gamec,(*gamec)->turn);
									
					} else {
		
						(*gamec)->turn->behaviour(gamec,(*gamec)->turn);
			
					}

				}

				walker = walker->next;

			}

		} else { /* MONSTER_FREEZE == 1 */

		}

		/* causes overflow after a while, 
		   so take care when dealing with a future cycle,
		   use the INCR_CYCLE(N) macro to get the cycle number N steps ahead.
		*/
		(*gamec)->cycle_cnt++;
	
	} else { /* GAME_FREEZE == 1 */

	}

}

GameControl* new_gamec() {
	
	GameControl* new = (GameControl*) g_malloc(sizeof(GameControl));

	new->map = (Map*)new_map();
	new->map_window = (MapWin*)new_map_window();
	new->inv_window = (InvWin*)new_inv_window();
	new->image_files = NULL;
	new->sprites = NULL;
	new->key_stack = NULL;
	new->items = NULL;
	new->local_items = NULL;
	new->monsters = NULL;
	new->local_monsters = NULL;
	new->turn = 0;
	new->game_state = 0;
	new->cycle_cnt = 1;

	return new;

}

void zap_gamec(GameControl* gamec) {

	if(gamec != NULL) {
		
		if(gamec->map != NULL) {
			
			zap_map(gamec->map);
		
		}

		if(gamec->map_window != NULL) {
			
			zap_map_window(gamec->map_window);
		
		}

		if(gamec->sprites != NULL) {
			
			g_list_foreach(gamec->sprites,(GFunc)zap_sprite,NULL);
			g_list_free(gamec->sprites);
			gamec->sprites = NULL;
		
		}
		
		if(gamec->image_files != NULL) {

			/* I don't free these strings because they're not dynamically allocated */
			g_list_free(gamec->image_files);
			gamec->image_files = NULL;

		}

		if(gamec->monsters != NULL) {

			g_list_foreach(gamec->monsters,(GFunc)zap_monster,NULL);
			g_list_free(gamec->monsters);
			gamec->monsters = NULL;
		
		}

		if(gamec->local_monsters != NULL) {

			g_list_free(gamec->local_monsters);
			gamec->local_monsters = NULL;
		
		}

		if(gamec->items != NULL) {

			g_list_foreach(gamec->items,(GFunc)zap_item,NULL);
			g_list_free(gamec->items);
			gamec->items = NULL;
		
		}

		if(gamec->local_items != NULL) {

			g_list_free(gamec->local_items);
			gamec->local_items = NULL;
		
		}
		
		g_free(gamec);
		gamec = NULL;

	}

}

void key_interface(GtkWidget* widget,GdkEventKey* event,GameControl** gamec) {

	gint n;

	if(event->type == GDK_KEY_PRESS) {

		switch(event->keyval) {

			case GDK_KP_Up:
			case GDK_KP_Page_Up:
			case GDK_KP_Right:
			case GDK_KP_Page_Down:
			case GDK_KP_Down:
			case GDK_KP_End:
			case GDK_KP_Left:
			case GDK_KP_Home:
				if((n = g_list_length((*gamec)->key_stack)) == 0) {
	
					(*gamec)->key_stack = g_list_append((*gamec)->key_stack,GINT_TO_POINTER(event->keyval));
	
				}
				break;
			/* this is a list of keyvals that's empty now,
			   because it's only used for commands... 
		           (e.g. M+Up+Right = move thing above you to the right)
		           and there are no commands yet.
			*/
			default:
				break;

		}

	}

}

void mouse_interface(GtkWidget* widget,GdkEventButton* event,GameControl** gamec) {

	if(event->type == GDK_BUTTON_PRESS) {

		if(event->button == 1) {
			g_print("%s\n",(MAP(((gint)(event->x)/TILE_SIZE)+AREA_OX,
			               ((gint)(event->y)/TILE_SIZE)+AREA_OY)->ios == NULL) ? 
			               ("no item there..."):
			               (MAP(((gint)(event->x)/TILE_SIZE)+AREA_OX,((gint)(event->y)/TILE_SIZE)+AREA_OY)->ios->name->str));

		} else if(event->button == 3) {

			if(MAP(((gint)(event->x)/TILE_SIZE)+AREA_OX,((gint)(event->y)/TILE_SIZE)+AREA_OY)->mos != NULL) {
	
				if(INV_WINDOW == NULL)
					open_inv_window(gamec);
				display_contains(gamec,MAP(((gint)(event->x)/TILE_SIZE)+AREA_OX,((gint)(event->y)/TILE_SIZE)+AREA_OY)->mos);

			}

		}

	}

}

void area_center_monster(GameControl** gamec,Monster* who) {

	gint new_area_ox,new_area_oy,i,j;

	if(who != NULL) {

		if(who->x - (((MAP_DRAWING_AREA->allocation.width)/TILE_SIZE)/2) >= 0) {
			
			if(who->x + (((MAP_DRAWING_AREA->allocation.width)/TILE_SIZE)/2)+1 < MAX_MAP_X) {

				new_area_ox = who->x - (((MAP_DRAWING_AREA->allocation.width)/TILE_SIZE)/2);
	
			} else {

				new_area_ox = (MAX_MAP_X - ((MAP_DRAWING_AREA->allocation.width)/TILE_SIZE))-1;

			}

		} else {

			new_area_ox = 0;

		}

		if(who->y - (((MAP_DRAWING_AREA->allocation.height)/TILE_SIZE)/2) >= 0) {

			if(who->y + (((MAP_DRAWING_AREA->allocation.height)/TILE_SIZE)/2)+1 < MAX_MAP_Y) {

				new_area_oy = who->y - (((MAP_DRAWING_AREA->allocation.height)/TILE_SIZE)/2);
	
			} else {

				new_area_oy = (MAX_MAP_Y - ((MAP_DRAWING_AREA->allocation.height)/TILE_SIZE))-1;

			}

		} else {

			new_area_oy = 0;

		}

		if((AREA_OX != new_area_ox) || (AREA_OY != new_area_oy)) {

			for(j=0;j<((MAP_DRAWING_AREA->allocation.height)/TILE_SIZE);j++)
			for(i=0;i<((MAP_DRAWING_AREA->allocation.width)/TILE_SIZE);i++)
				add_update(gamec,new_area_ox+i,new_area_oy+j);

			AREA_OX = new_area_ox;
			AREA_OY = new_area_oy;

		}

	}

}

void freeze_game(GameControl** gamec) {

	(*gamec)->game_state = (CHK((*gamec)->game_state,GAME_FREEZE)) ?
	                       (CLR((*gamec)->game_state,GAME_FREEZE)) :
	                       (SET((*gamec)->game_state,GAME_FREEZE));

}

void freeze_monsters(GameControl** gamec) {

	(*gamec)->game_state = (CHK((*gamec)->game_state,MONSTERS_FREEZE)) ?
	                       (CLR((*gamec)->game_state,MONSTERS_FREEZE)) :
	                       (SET((*gamec)->game_state,MONSTERS_FREEZE));

}
/* FIXME: remove this function, it's only a temporary debug-play function
   it only works well when there's one player character and one non player character in game */
void change_player(GameControl** gamec) {

	GList* walker = (*gamec)->monsters;
	gboolean b = TRUE;	

	while(walker != NULL) {

		if(walker->data != NULL) {
			if(CHK(((Monster*)walker->data)->properties,IS_PLAYER)) {
	
				((Monster*)walker->data)->properties = (CLR(((Monster*)walker->data)->properties,IS_PLAYER));
			
			} else if(b) {

				((Monster*)walker->data)->properties = (SET(((Monster*)walker->data)->properties,IS_PLAYER));
				b = FALSE;

			} else return;

		}

		walker = walker->next;

	}

}