#ifndef QUEST_GLOBAL
#define QUEST_GLOBAL

/* debugging, used in: -all error reports- */
#define VERBOSE_ERRORS		(0)

/* game version, used in: init_widgets() */
#define QUEST_WINDOW_TITLE 	"gQuest - 0.0.19"
#define ABOUT_STRING		"gQuest - 0.0.19\n\n(C) 1999 Willem van Hage\n\nA Role Playing Game for the open community\nhomepage: http://gene.wins.uva.nl/~wrvhage\nReleased under terms of the GNU Public License.\n"

/* game cycle delay time in milliseconds, used in: init_widgets() */
#define GAME_CYCLE_TIME		(75)

/* map dimensions, used in: Map.cell MAKE SURE THESE VALUES FIT IN 16 BITS!*/
#define MAX_MAP_X		(30) 
#define MAX_MAP_Y		(30)
		
/* the size of a tile in pixels... 
   could be calculated from the tile images,
   but I prefer not to do so, because image sizes might
   differ from tile sizes, just like big monsters only use one image */
#define TILE_SIZE		(24)

/* struct types, used in: Coords, Item, Monster */
typedef enum {
	
	COORDS = 0,
	ITEM,
	MONSTER

} StructType;

/* memory usage, used in: Memory */
typedef enum {

	UNKNOWN_MEM = -1,
	INT_MEM = 0,
	COORDS_MEM,
	ITEM_MEM,
	MONSTER_MEM

} MemoryType;

/* space types, used in: Monster.access, Space.kind */
typedef enum {
	
	UNKNOWN_KIND	= 1 << 0,
	MOUNTAIN 	= 1 << 1,
	WATER		= 1 << 2,
	GRASS		= 1 << 3,
	DIRT		= 1 << 4,
	STREET		= 1 << 5,
	LAVA		= 1 << 6,
	ICE		= 1 << 7,
	SAND		= 1 << 8

} SpaceKind;

/* directions, used in:	-move functions- */
typedef enum {
	
	UP = 1,
	UP_RIGHT,
	RIGHT,
	DOWN_RIGHT,
	DOWN,
	DOWN_LEFT,
	LEFT,
	UP_LEFT

} Direction;

/* pose types, used in:	Item.state */
typedef enum {

	BROKEN = 0,
	DAMAGED_1,
	DAMAGED_2,
	DAMAGED_3,
	DAMAGED_4,
	OK_1,
	OK_2,
	OK_3,
	OK_4,
	ON_FIRE_1,
	ON_FIRE_2,
	ON_FIRE_3,
	ON_FIRE_4,

	/* direction_action_foot */

	DOWN_WALK_LEFT_1,
	DOWN_WALK_RIGHT_1,
	LEFT_WALK_LEFT_1,
	LEFT_WALK_RIGHT_1,
	UP_WALK_LEFT_1,
	UP_WALK_RIGHT_1,
	RIGHT_WALK_LEFT_1,
	RIGHT_WALK_RIGHT_1,
	DOWN_IDLE_1,
	LEFT_IDLE_1,
	UP_IDLE_1,
	RIGHT_IDLE_1,
	DOWN_ATTACK_1,
	LEFT_ATTACK_1,
	UP_ATTACK_1,
	RIGHT_ATTACK_1

} State;

/* inventory locations, used in: -dealing with Item.contains- */
typedef enum {

	RIGHT_HAND = 1,
	LEFT_HAND,
	BELT,
	BODY,
	HEAD,
	FEET,
	BACK

} ContainsLocation;

/* hacky way to describe a monster's state, used in: Monster.state_flags */
typedef enum {

	/* a direction is defined as:

	UP = 0,
	UP_RIGHT,
	RIGHT,
	DOWN_RIGHT,
	DOWN,
	DOWN_LEFT,
	LEFT,
	UP_LEFT,	is binary 0111 
			so the next bit it the fourth bit,
			that's 1000 (1 << 3) */

	LEFTY_BIT	= 1 << 3,
	WALK_BIT	= 1 << 4,
	ATTACK_BIT	= 1 << 5

} StateFlags;

/* animation frame names for spaces, used in: set_space() */
typedef enum {

	FIRST_FRAME = 1,
	SECOND_FRAME,
	THIRD_FRAME,
	FOURTH_FRAME

} SpaceFrame;

/* sorts, used in: Item.sort */
typedef enum {

	UNKNOWN_SORT = 0,
	UNARMED = 0,
	RUG = 1,
	BOULDER,
	TREE,
	BARREL,
	FIRE,
	FUNGUS,
	SWORD,
	CLUB,

	HUMAN,
	ELF,
	DWARF,
	GNOME,
	ORC,
	GOBLIN,
	KOBOLD,
	OGRE,
	BEAR,
	DOMESTICAL,
	SYLVAN,
	ELEMENTAL,
	DRAGON

} Sort;

/* sub_sorts, used in: Item.sub_sort */
typedef enum {

	UNKNOWN_SUB_SORT = 0,
	RED_RUG,
	PURPLE_RUG,
	BIG_BOULDER,
	MOSSY_BOULDER,
	BIG_TREE,
	SMALL_TREE,
	LIGHT_BARREL,
	DARK_BARREL,
	CAMP_FIRE,
	MUSHROOM_RING_FUNGUS,
	BROAD_SWORD,
	SHORT_SWORD,
	SPIKED_CLUB,

	KNIGHT_HUMAN,
	ARCHER_ELF,
	BLACKSMITH_DWARF,
	TINKER_GNOME,
	WARRIOR_ORC,
	GREEN_GOBLIN,
	SMELLY_KOBOLD,
	BRUTE_OGRE,
	BROWN_BEAR,
	CAT_DOMESTICAL,
	SHEEP_DOMESTICAL,
	UNICORN_SYLVAN,
	DRYAD_SYLVAN,
	AIR_ELEMENTAL,
	RED_DRAGON

} SubSort;


/* item properties, used in: Item.properties */
typedef enum {

	IS_MOVABLE	= 1 << 0,
	IS_BASHABLE	= 1 << 1,
	IS_STACKABLE	= 1 << 2,
	IS_FLAMABLE	= 1 << 3,
	IS_TREADABLE	= 1 << 4,
	IS_PORTABLE	= 1 << 5

} Properties;

/* unique monster properties, used in: Monster.m_properties */
typedef enum {

	IS_PLAYER	= 1 << 0,
	IS_DEAD		= 1 << 1,
	IS_HOSTILE	= 1 << 2,
	IS_SLEEPING	= 1 << 3,
	IS_DIZZY	= 1 << 4,
	IS_INVISIBLE	= 1 << 5,
	IS_PETRIFIED	= 1 << 6

} MProperties;

/* does damage occasions, used in: Item.dmg_sort */
typedef enum {

	NEVER		= 1 << 0,
	CONTACT		= 1 << 1,
	WIELD		= 1 << 2,
	MISSILE		= 1 << 3

} DamageSort;

/* x and y composition of directions, used in: -nothing yet- */
typedef enum {

	NORTH_X		= 0,
	NORTH_EAST_X	= 1,
	EAST_X		= 1,
	SOUTH_EAST_X	= 1,
	SOUTH_X		= 0,
	SOUTH_WEST_X	= -1,
	WEST_X		= -1,
	NORTH_WEST_X	= -1,
	NORTH_Y		= -1,
	NORTH_EAST_Y	= -1,
	EAST_Y		= 0,
	SOUTH_EAST_Y	= 1,
	SOUTH_Y		= 1,
	SOUTH_WEST_Y	= 1,
	WEST_Y		= 0,
	NORTH_WEST_Y	= -1

} DirectionXY;

/* game control flags, used in: GameControl.game_state */
typedef enum {
	
	COMBAT_MODE	= 1 << 0,
	GAME_FREEZE	= 1 << 1,	/* complete game freeze */
	MONSTERS_FREEZE	= 1 << 2	/* only monsters freeze 
					   (to have time to work out your command in command mode) */

} GameState;

/* sprite id's, used in: IMAGE(), PIXMAP(), MASK() */
typedef enum {

	INTRO_SCREEN = 0,
	QUEST_LOGO,
	UNKNOWN_SPACE_SPRITE,
	GRASS_1,
	GRASS_2,
	DIRT_1,
	DIRT_2,
	UNKNOWN_ITEM_SPRITE,
	CAMPFIRE_DOUSED,
	CAMPFIRE_BURNING_1,
	CAMPFIRE_BURNING_2,
	CAMPFIRE_BURNING_3,
	CAMPFIRE_BURNING_4,
	RED_RUG_1,
	PURPLE_RUG_1,
	BIG_BOULDER_1,
	THREE_BOULDERS_1,
	TREE_1,
	TWO_TREES_1,
	BIG_TREE_1,
	WOOD_SHREDS_1,
	DARK_BARREL_1,
	LIGHT_BARREL_1,
	SMASHED_BARREL_1,
	MUSHROOM_RING_1,
	SHORT_SWORD_1,
	SPIKED_CLUB_1,
	OGRE_DOWN_WALK_LEFT_1,
	OGRE_DOWN_WALK_RIGHT_1,
	OGRE_UP_WALK_LEFT_1,
	OGRE_UP_WALK_RIGHT_1,
	OGRE_LEFT_WALK_LEFT_1,
	OGRE_LEFT_WALK_RIGHT_1,
	OGRE_RIGHT_WALK_LEFT_1,
	OGRE_RIGHT_WALK_RIGHT_1,
	KNIGHT_DOWN_WALK_LEFT_1,
	KNIGHT_DOWN_WALK_RIGHT_1,
	KNIGHT_UP_WALK_LEFT_1,
	KNIGHT_UP_WALK_RIGHT_1,
	KNIGHT_LEFT_WALK_LEFT_1,
	KNIGHT_LEFT_WALK_RIGHT_1,
	KNIGHT_RIGHT_WALK_LEFT_1,
	KNIGHT_RIGHT_WALK_RIGHT_1

} SpriteName;

/* quick image retriever, used in: wherever you need an image */
#define SPRITELIST		((*gamec)->sprites)
#define IMAGE(N)		(((Sprite*) g_list_nth_data(SPRITELIST,N))->image)
#define PIXMAP(N)		(((Sprite*) g_list_nth_data(SPRITELIST,N))->pixmap)
#define MASK(N)			(((Sprite*) g_list_nth_data(SPRITELIST,N))->mask)

/* future functions, used in: -wherever you need (gamec->cycle_cnt)+n- */
#define CYCLE_CNT		((*gamec)->cycle_cnt)
#define CYCLE_CNT_MAX		(256)
#define INCR_CYCLE(N)		((CYCLE_CNT+N)%CYCLE_CNT_MAX)

/* mask operators, used in: -wherever you set,clr,chk a mask- */
#define SET(X,B)		(X | B)
#define CLR(X,B)		(X & ~B)
#define CHK(X,B)		(((X & B) == 0) ? 0 : 1)
#define CNG(X,B)		((X | B) & (-1 & B))
#define CNG_W_MASK(X,B,M)	((~M & X) | (M & B))

/* widgets and other fields, used in: init_widgets(), -drawing functions- */
#define MAP_WINDOW		((*gamec)->map_window->window)
#define MAP_BOX			((*gamec)->map_window->box)
#define MAP_MENU		((*gamec)->map_window->menu)
#define MAP_DRAWING_AREA	((*gamec)->map_window->drawing_area)
#define MAP_EVENT_BOX		((*gamec)->map_window->event_box)
#define MAP_AREA		((*gamec)->map_window->area)
#define MAP_GC			((*gamec)->map_window->gc)
#define MAP_UPDATE		((*gamec)->map_window->update)
#define INV_WINDOW		((*gamec)->inv_window->window)
#define INV_SCROLLED_WIN	((*gamec)->inv_window->scrolled_win)
#define INV_BOX			((*gamec)->inv_window->box)
#define INV_MENU		((*gamec)->inv_window->menu)
#define INV_CLIST		((*gamec)->inv_window->clist)
#define INV_WHO			((*gamec)->inv_window->who)

/* the viewing area's origins, used in:	make_area() */
#define AREA_OX			((*gamec)->map_window->area_ox)
#define AREA_OY			((*gamec)->map_window->area_oy)

/* space retrieval, used in: -map functions- */
#define MAP(X,Y)		((*gamec)->map->cell[X][Y])

/* easy viewing area dimension macro's */
#define XSPACES			(((MAP_DRAWING_AREA->allocation.width)/TILE_SIZE)+1)
#define YSPACES			(((MAP_DRAWING_AREA->allocation.height)/TILE_SIZE)+1)

/* can_move_X return values, used in: can_move_item() can_move_monster() */
typedef enum {

	YES_CLEAR = 0,
	NO_ITEM,
	NO_MONSTER,
	NO_ACCESS

} CanMove;

#endif /* QUEST_GLOBAL */
