#ifndef QUEST_TYPES
#define QUEST_TYPES

#include <gtk/gtk.h>
#include <gdk_imlib.h>
#include "global.h"


/* SPRITES */

#define SPRITE_DECL												\
														\
gint32		key;		/* makes it possible to knead structs with a DLL into a hash-like key list */	\
GdkImlibImage*	image;		/* the image to derive the mask and pixmap from, loaded first */		\
GdkPixmap*	pixmap;		/* the pixmap that's used in the game */					\
GdkBitmap*	mask;		/* the mask that belongs to the pixmap, ready for use */

typedef struct Sprite {

	SPRITE_DECL

} Sprite;


/* COORDINATES, ITEMS AND MONSTERS */

#define COORDS_DECL												\
														\
gint32		key;		/* makes it possible to knead structs with a DLL into a hash-like key list */	\
gint8		type;		/* Coords,Item or Monster... enumeration */					\
gint32		x;		/* the absolute x on the map */							\
gint32		y;		/* the absolute y on the map...							\
				   raw, absolute map coordinates,						\
				   big items and monsters (with width and/or height > 1)			\
				   have their x and y set to their upperleft square */

#define ITEM_DECL												\
														\
COORDS_DECL													\
														\
GString*	name;		/* a description when looking at something */					\
gint8		width;		/* the map width measured in squares */						\
gint8		height;		/* the map height measured in squares */					\
GList*		sprites;	/* uses enum from global.h as keys, SpritesNr as values */			\
gboolean	animates;	/* when FALSE don't even bother animating... */					\
gint16		state;		/* the current sprites key value to use when drawing the object */		\
guint16		access;		/* a mask of all accessible Space types put together with an OR operation */	\
guint16		properties;	/* what you can do with it, uses the IS_<foo>ABLE #defines from global.h */	\
guint16		sort;		/* the group it belongs to SWORD, EVIL_MONSTER */				\
guint16		sub_sort;	/* the exact sort, BROAD_SWORD, OGRE */						\
gint32		weight;		/* weight in cups of sugar ;) */						\
gint16		max_hp;		/* number of hitpoints in perfect condition */					\
gint16		cur_hp;		/* current hitpoints with possible damage subtracted */				\
gint16		dmg_sort;	/* item causes damage on this occasion,CONTACT,WIELD etc, enum from global.h */	\
gint16		min_dmg;	/* touch it and it'll at least cause min_dmg hitpoints of damage */		\
gint16		max_dmg;	/* and at most max_dmg hitpoints of damage */					\
gint16		use_cost;	/* number of action points required to use the item */				\
gint16		armor;		/* percentile invulnerability */						\
gint16		max_weight;	/* maximum weight the object can contain */					\
GList*		contains;	/* other things inside the object, uses #defined keys that indicate where */	\
struct Item*	inside;		/* points to the Item the object is in, otherwise NULL */			\
struct Item*	stacked;	/* what's on top of the object? barrels on rugs, riders on horses...		\
				   yes, it's allowed to typecast a Monster rider to an item to put		\
				   him on a horse, and you can still access the monster abilities */

#define MONSTER_DECL												\
														\
ITEM_DECL													\
														\
guint16		state_flags;		/* describes the state of the monster in analysable bit flags */	\
GList*		memory;			/* memory space to store data used in behaviour functions */		\
GList*		weapon_skills;		/* uses the Item.sort field as key and percentile skills */		\
GList*		non_weapon_skills;	/* uses global.h #defines as keys and percentile skills */		\
guint16		m_properties;		/* mask of IS_<foo> #defines, like IS_DEAD and IS_INVISIBLE */		\
gint16		agression;		/* percentile chance the monster will get hostile when provoked */	\
gint16		move_cost;		/* action points needed for a single move, determines speed */		\
gint16		cost_cnt;		/* shows when the monster can act again (when cost_cnt == 0) */		\
void		(*behaviour)();		/* pointer to a function that gets called once every turn */

typedef struct Coords {

	COORDS_DECL

} Coords;

typedef struct Item {

	ITEM_DECL

} Item;

typedef struct Monster {

	MONSTER_DECL

} Monster;


/* MEMORY AND SKILLS */

#define MEMORY_DECL												\
														\
gint32		key;												\
gint16		kind;												\
gint		int_mem;											\
gpointer	pointer_mem;

#define SKILL_DECL												\
														\
gint32		key;												\
gint16		skill;												

#define SPRITE_NR_DECL												\
														\
gint32		key;												\
gint16		nr;

typedef struct Memory {

	MEMORY_DECL

} Memory;

typedef struct Skill {

	SKILL_DECL

} Skill;

typedef struct SpriteNr {

	SPRITE_NR_DECL

} SpriteNr;


/* SPACES AND THE MAP */

#define SPACE_DECL												\
														\
gint32		key;		/* makes it possible to knead structs with a DLL into a hash-like key list */	\
gint16		kind;		/* GRASS, DIRT etc... #defines from global.h */					\
gint16		variation;	/* what pixmap to use, lighter grass, darker grass */				\
GList*		sprites;	/* uses enum from global.h as keys for the SpriteNr's that it uses as values */	\
gboolean	animates;	/* when FALSE don't even bother animating the space... */			\
guint16		sprite_cur;	/* the key of the sprite that should be used when drawing the space */		\
Monster*	mos;		/* pointer to the monster on the space */					\
Item*		ios;		/* pointer to the item on the space, stacked items use the stacked * in Item */

#define MAP_DECL												\
														\
GString*	name;				/* guess... */							\
Space*		cell[MAX_MAP_X][MAX_MAP_Y];	/* a matrix of spaces that makes up the real map 		\
						   MAX_MAP_X and MAX_MAP_Y are #defines from global.h */

typedef struct Space {

	SPACE_DECL

} Space;

typedef struct Map {

	MAP_DECL

} Map;


/* WINDOWS AND GAME CONTROL */

#define MAP_WIN_DECL												\
														\
GtkWidget*	window;		/* pointer to the gtk_window to use */						\
GtkWidget*	box;		/* pointer to the vbox to put the menu and the drawing_area in */		\
GtkWidget*	menu;		/* pointer to the menu bar to use */						\
GtkWidget*	drawing_area;	/* pointer to the drawing area to draw the visible area of the map in */	\
GtkWidget*	event_box;	/* to connect mouse and keyboard events to the drawing area */			\
GdkPixmap*	area;		/* the GDK pixmap where the real drawing action is... exciting stuff... */	\
GdkGC*		gc;		/* the Graphical Context I use to put the masks in for sprite transparency */	\
GList*		update;		/* a list with Coords of all the map coords that need updated sprites */	\
gint32		area_ox;	/* the absolute map x coordinate where the visible area of the map starts */	\
gint32		area_oy;	/* the absolute map x coordinate where the visible area of the map starts */

#define INV_WIN_DECL												\
														\
GtkWidget*	window;		/* pointer to the gtk_window for the inventory window */			\
GtkWidget*	scrolled_win;	/* pointer to the scrolled window to put the clist in */			\
GtkWidget*	box;		/* pointer to the vbox to put the menu and the drawing_area in */		\
GtkWidget*	menu;		/* pointer to the menu bar to use */						\
GtkWidget*	clist;		/* pointer to the Clist that shows the inventory */				\
Monster*	who;		/* pointer to a monster or item whose contains field is displayed */

#define GAME_CONTROL_DECL											\
														\
Map*		map;		/* pointer to the current map that is used for the game */			\
MapWin*		map_window;	/* pointer to the map window struct with data for the current map window */	\
InvWin*		inv_window;	/* pointer to the inventory window struct that shows the current inventory */	\
GList*		image_files;	/* a list of filenames of pictures to load from disk */				\
GList*		sprites;	/* the list of sprites to use, that's images,pixmaps and masks */		\
GList*		key_stack;	/* a list of all the keys that have been pressed this turn */			\
GList*		items;		/* a list of all active items */						\
GList*		local_items;	/* all items near the player... items that can have influence on gameplay */	\
GList*		monsters;	/* a list of all active monsters */						\
GList*		local_monsters;	/* all monsters near the player... only these monsters get a turn */		\
Monster*	turn;		/* pointer to the monster whose turn it is */					\
guint16		game_state;	/* a mask of bits that show in what state the game is now... a quit bit etc */	\
guint8		cycle_cnt;	/* the current "game time" */

typedef struct MapWin {

	MAP_WIN_DECL

} MapWin;

typedef struct InvWin {

	INV_WIN_DECL

} InvWin;

typedef struct GameControl {

	GAME_CONTROL_DECL

} GameControl;


#endif /* QUEST_TYPES */